@@ -4,17 +4,18 @@ Copyright 2021 Objectiv B.V.
 import json
 import operator
 from functools import reduce
-from abc import abstractmethod
 from typing import Dict, Union, TYPE_CHECKING, Tuple, cast, Optional, List, Any, TypeVar, Generic
 
 from sqlalchemy.engine import Dialect
 
+from bach import SortColumn
 from bach.series import Series
-from bach.expression import Expression, join_expressions
+from bach.expression import Expression
 from bach.series.series import WrappedPartition, ToPandasInfo
 from bach.sql_model import BachSqlModel
-from bach.types import DtypeOrAlias, StructuredDtype, AllSupportedLiteralTypes, Dtype
+from bach.types import DtypeOrAlias, StructuredDtype, AllSupportedLiteralTypes
 from sql_models.constants import DBDialect
+from sql_models.model import Materialization
 from sql_models.util import quote_string, is_postgres, DatabaseNotSupportedException, is_bigquery
 
 if TYPE_CHECKING:
@@ -55,15 +56,15 @@ class SeriesJson(Series):
 
     Examples:
 
-     .. testsetup:: jsonb
+     .. testsetup:: json
         :skipif: engine is None
 
         data = ['["a","b","c"]', '["d","e","f","g"]', '[{"h":"i","j":"k"},{"l":["m","n","o"]},{"p":"q"}]']
         pdf = pd.DataFrame(data=data, columns=['jsonb_column'])
         df = DataFrame.from_pandas(engine, pdf, convert_objects=True)
-        df['jsonb_column'] = df.jsonb_column.astype('jsonb')
+        df['jsonb_column'] = df.jsonb_column.astype('json')
 
-    .. doctest:: jsonb
+    .. doctest:: json
         :skipif: engine is None
 
         >>> pdf
@@ -72,11 +73,11 @@ class SeriesJson(Series):
         1                                  ["d","e","f","g"]
         2  [{"h":"i","j":"k"},{"l":["m","n","o"]},{"p":"q"}]
 
-    .. doctest:: jsonb
+    .. doctest:: json
         :skipif: engine is None
 
         >>> df = DataFrame.from_pandas(engine, pdf, convert_objects=True)
-        >>> df['jsonb_column'] = df.jsonb_column.astype('jsonb')
+        >>> df['jsonb_column'] = df.jsonb_column.astype('json')
         >>> # load some json strings and convert them to jsonb type
         >>> # slice and show with .head()
         >>> df.jsonb_column.json[:2].head()
@@ -86,7 +87,7 @@ class SeriesJson(Series):
         2    [{'h': 'i', 'j': 'k'}, {'l': ['m', 'n', 'o']}]
         Name: jsonb_column, dtype: object
 
-    .. doctest:: jsonb
+    .. doctest:: json
         :skipif: engine is None
 
         >>> df.jsonb_column.json[1].head()
@@ -96,7 +97,7 @@ class SeriesJson(Series):
         2    {'l': ['m', 'n', 'o']}
         Name: jsonb_column, dtype: object
 
-    .. doctest:: jsonb
+    .. doctest:: json
         :skipif: engine is None
 
         >>> # selecting from objects is done by entering a key:
@@ -112,25 +113,25 @@ class SeriesJson(Series):
     to the objects in a json array is returned for the `.json[]` selector. A match is when all key/value pairs
     of the dict are found in an object. This can be used for selecting a subset of a json array with objects.
 
-    .. doctest:: jsonb
+    .. doctest:: json
         :skipif: engine is None
 
         >>> # selecting from arrays by searching objects in the array.
         >>> df.jsonb_column.json[:{"j":"k"}].head()
         _index_0
-        0                      None
-        1                      None
+        0                        []
+        1                        []
         2    [{'h': 'i', 'j': 'k'}]
         Name: jsonb_column, dtype: object
 
-    .. doctest:: jsonb
+    .. doctest:: json
         :skipif: engine is None
 
         >>> # or:
         >>> df.jsonb_column.json[{"l":["m","n","o"]}:].head()
         _index_0
-        0                                    None
-        1                                    None
+        0                                      []
+        1                                      []
         2    [{'l': ['m', 'n', 'o']}, {'p': 'q'}]
         Name: jsonb_column, dtype: object
     """
@@ -165,9 +166,10 @@ class SeriesJson(Series):
     @classmethod
     def supported_literal_to_expression(cls, dialect: Dialect, literal: Expression) -> Expression:
         if is_postgres(dialect):
-            return Expression.construct(f'cast({{}} as {cls.get_db_dtype(dialect)})', literal)
+            return super().supported_literal_to_expression(dialect=dialect, literal=literal)
         if is_bigquery(dialect):
-            return literal
+            from bach import SeriesString
+            return SeriesString.supported_literal_to_expression(dialect=dialect, literal=literal)
         raise DatabaseNotSupportedException(dialect)
 
     @classmethod
@@ -279,8 +281,7 @@ class SeriesJsonPostgres(SeriesJson):
                  name: str,
                  expression: Expression,
                  group_by: 'GroupBy',
-                 sorted_ascending: Optional[bool],
-                 index_sorting: List[bool],
+                 order_by: Optional[List[SortColumn]],
                  instance_dtype: StructuredDtype,
                  **kwargs):
 
@@ -290,8 +291,7 @@ class SeriesJsonPostgres(SeriesJson):
                          name=name,
                          expression=Expression.construct(f'cast({{}} as jsonb)', expression),
                          group_by=group_by,
-                         sorted_ascending=sorted_ascending,
-                         index_sorting=index_sorting,
+                         order_by=order_by,
                          instance_dtype=instance_dtype,
                          **kwargs)
 
@@ -313,6 +313,7 @@ class SeriesJsonPostgres(SeriesJson):
             node_name='manual_materialize',
             limit: Any = None,
             distinct: bool = False,
+            materialization: Union[Materialization, str] = Materialization.CTE
     ):
         """
         Instance of this Series cannot be materialized, as the base expression is not just the column name
@@ -414,6 +415,23 @@ class JsonAccessor(Generic[TSeriesJson]):
 
         return self._implementation.array_contains(item)
 
+    def flatten_array(self) -> Tuple['TSeriesJson', 'SeriesInt64']:
+        """
+        Converts elements in an array into a set of rows.
+
+        Since the operation might destroy the order of the elements,
+        a series containing the offset is also returned.
+
+        :returns: Tuple with SeriesJson (element from the array) and SeriesInt64 (offset of the element)
+
+        .. note::
+            Both returned series objects will share same base node, but this node is not
+            the same as the unflatten Series.
+
+        This assumes the top-level item in the json is an array.
+        """
+        return self._implementation.flatten_array()
+
 
 class JsonBigQueryAccessorImpl(Generic[TSeriesJson]):
     """
@@ -594,6 +612,11 @@ class JsonBigQueryAccessorImpl(Generic[TSeriesJson]):
             .copy_override(expression=expression)\
             .copy_override_type(SeriesBoolean)
 
+    def flatten_array(self) -> Tuple['TSeriesJson', 'SeriesInt64']:
+        """ For documentation, see implementation in class :class:`JsonAccessor` """
+        from bach.series.array_operations.flattening import BigQueryArrayFlattening
+        return BigQueryArrayFlattening(self._series_object)()
+
 
 class JsonPostgresAccessorImpl(Generic[TSeriesJson]):
     """
@@ -708,3 +731,8 @@ class JsonPostgresAccessorImpl(Generic[TSeriesJson]):
     def array_contains(self, item: Union[int, float, bool, str, None]) -> 'SeriesBoolean':
         """ For documentation, see implementation in class :class:`JsonAccessor` """
         return self._series_object._comparator_operation([item], "@>")
+
+    def flatten_array(self) -> Tuple['TSeriesJson', 'SeriesInt64']:
+        """ For documentation, see implementation in class :class:`JsonAccessor` """
+        from bach.series.array_operations.flattening import PostgresArrayFlattening
+        return PostgresArrayFlattening(self._series_object)()
