@@ -47,10 +47,6 @@ class SeriesList(Series):
 
     supported_value_types = (list, )
 
-    @classmethod
-    def supported_literal_to_expression(cls, dialect: Dialect, literal: Expression) -> Expression:
-        return literal
-
     @classmethod
     def supported_value_to_literal(
         cls,
@@ -83,7 +79,7 @@ class SeriesList(Series):
         cls,
         base: 'DataFrameOrSeries',
         value: List[Any],
-        name: str,
+        name: str = 'new_series',
         dtype: Optional[StructuredDtype] = None
     ) -> 'SeriesList':
         """
@@ -137,8 +133,7 @@ class SeriesList(Series):
             name=name,
             expression=expression,
             group_by=None,
-            sorted_ascending=None,
-            index_sorting=[],
+            order_by=[],
             instance_dtype=dtype
         )
         return result
