@@ -1,18 +1,20 @@
 """
 Copyright 2021 Objectiv B.V.
 """
+import math
 from abc import ABC
 from typing import cast, Union, TYPE_CHECKING, Optional, List, Tuple
 
 import numpy
 from sqlalchemy.engine import Dialect
 
+from bach import DataFrameOrSeries
 from bach.series import Series
 from bach.expression import Expression, AggregateFunctionExpression
 from bach.series.series import WrappedPartition
 from bach.types import StructuredDtype
 from sql_models.constants import DBDialect
-from sql_models.util import is_postgres, is_bigquery, DatabaseNotSupportedException
+from sql_models.util import is_postgres, is_bigquery, DatabaseNotSupportedException, is_athena
 
 if TYPE_CHECKING:
     from bach.series import SeriesBoolean, SeriesNumericInterval
@@ -54,6 +56,10 @@ class SeriesAbstractNumeric(Series, ABC):
 
         :param decimals: The amount of decimals to round to
         """
+        if is_athena(self.engine):
+            return self.copy_override(
+                expression=Expression.construct(f'round({{}}, {decimals})', self)
+            )
         return self.copy_override(
             expression=Expression.construct(f'round(cast({{}} as numeric), {decimals})', self)
         )
@@ -235,22 +241,29 @@ class SeriesInt64(SeriesAbstractNumeric):
     dtype_aliases = ('integer', 'bigint', 'i8', int, numpy.int64, 'int32')
     supported_db_dtype = {
         DBDialect.POSTGRES: 'bigint',
+        DBDialect.ATHENA: 'bigint',
         DBDialect.BIGQUERY: 'INT64'
     }
     supported_value_types = (int, numpy.int64, numpy.int32)
 
     @classmethod
     def supported_literal_to_expression(cls, dialect: Dialect, literal: Expression) -> Expression:
-        if is_postgres(dialect):
+        # We override the parent class here because integers are really common, and we don't strictly need
+        # to cast all of them for BigQuery. Not casting integer literals greatly improves the readability of
+        # the generated SQL.
+
+        if is_postgres(dialect) or is_athena(dialect):
             # A stringified integer is a valid integer or bigint literal, depending on the size. We want to
             # consistently get bigints, so always cast the result
-            # See the section on numeric constants in the Postgres documentation
+            # PG: See the section on numeric constants in the Postgres documentation
             # https://www.postgresql.org/docs/14/sql-syntax-lexical.html#SQL-SYNTAX-CONSTANTS
-            return Expression.construct(f'cast({{}} as {cls.get_db_dtype(dialect)})', literal)
+            # Athena: experimentally established that the integer typing mimicks Postgres's behaviour
+            return super().supported_literal_to_expression(dialect=dialect, literal=literal)
         if is_bigquery(dialect):
             # BigQuery has only one integer type, so there is no confusion between 32-bit and 64-bit integers
             # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-types#integer_type
             # https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical#integer_literals
+            # Additionally, we don't even need to cast NULL, because on BigQuery that's an INT64 by default.
             return literal
         raise DatabaseNotSupportedException(dialect)
 
@@ -287,6 +300,13 @@ class SeriesInt64(SeriesAbstractNumeric):
             # Postgres expects the argument to a bitshift to be a regular int, not bigint.
             return self._arithmetic_operation(other, 'rshift', '({}) >> cast({} as int)',
                                               other_dtypes=tuple(['int64']))
+        if is_athena(self.engine):
+            # Latest version of prestodb supports bitwise_logical_shift_right() [1], but the version that
+            # Athena uses, does not [2]. So we do some hackish thing here
+            # [1] https://prestodb.io/docs/current/functions/bitwise.html
+            # [2] https://prestodb.io/docs/0.217/functions/bitwise.html
+            return self._arithmetic_operation(other, 'rshift', 'cast(({}) / power(2, {}) as bigint)',
+                                              other_dtypes=tuple(['int64']))
         return self._arithmetic_operation(other, 'rshift', '({}) >> ({})', other_dtypes=tuple(['int64']))
 
     def __lshift__(self, other):
@@ -294,6 +314,11 @@ class SeriesInt64(SeriesAbstractNumeric):
             # Postgres expects the argument to a bitshift to be a regular int, not bigint.
             return self._arithmetic_operation(other, 'lshift', '({}) << cast({} as int)',
                                               other_dtypes=tuple(['int64']))
+        if is_athena(self.engine):
+            # hack around missing shift operator
+            return self._arithmetic_operation(other, 'lshift', 'cast(({}) * power(2, {}) as bigint)',
+                                              other_dtypes=tuple(['int64']))
+
         return self._arithmetic_operation(other, 'lshift', '({}) << ({})', other_dtypes=tuple(['int64']))
 
     def sum(self, partition: WrappedPartition = None, skipna: bool = True, min_count: int = None, **kwargs):
@@ -332,6 +357,7 @@ class SeriesFloat64(SeriesAbstractNumeric):
     dtype_aliases = ('float', 'double', 'f8', float, numpy.float64, 'double precision')
     supported_db_dtype = {
         DBDialect.POSTGRES: 'double precision',
+        DBDialect.ATHENA: 'double',
         DBDialect.BIGQUERY: 'FLOAT64'
     }
     supported_value_types = (float, numpy.float64)
@@ -354,10 +380,6 @@ class SeriesFloat64(SeriesAbstractNumeric):
     # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-types#floating_point_types
     # https://cloud.google.com/bigquery/docs/reference/standard-sql/lexical#floating_point_literals
 
-    @classmethod
-    def supported_literal_to_expression(cls, dialect: Dialect, literal: Expression) -> Expression:
-        return Expression.construct(f'cast({{}} as {cls.get_db_dtype(dialect)})', literal)
-
     @classmethod
     def supported_value_to_literal(
         cls,
@@ -365,6 +387,15 @@ class SeriesFloat64(SeriesAbstractNumeric):
         value: Union[float, numpy.float64],
         dtype: StructuredDtype
     ) -> Expression:
+        if is_athena(dialect) and not math.isfinite(value):
+            # see https://prestodb.io/docs/0.217/functions/math.html, under 'Floating Point functions'
+            if math.isnan(value):
+                return Expression.raw('nan()')
+            if math.isinf(value):
+                if value > 0:
+                    return Expression.raw('infinity()')
+                else:
+                    return Expression.raw('-infinity()')
         return Expression.string_value(str(value))
 
     @classmethod
@@ -374,3 +405,48 @@ class SeriesFloat64(SeriesAbstractNumeric):
         if source_dtype not in ['int64', 'string']:
             raise ValueError(f'cannot convert {source_dtype} to float64')
         return Expression.construct(f'cast({{}} as {cls.get_db_dtype(dialect)})', expression)
+
+    @classmethod
+    def random(cls, base: DataFrameOrSeries) -> 'SeriesFloat64':
+        """
+        Create a new Series object with an expression, that will evaluate to a random float in the range
+        [0, 1) for each row.
+
+        :param base: DataFrame or Series from which the newly created Series' engine, base_node and index
+            parameters are copied.
+
+        .. warning::
+            The returned Series has a non-deterministic expression, it will give a different result each
+            time it is evaluated by the database.
+
+        The non-deterministic expression can have some unexpected consequences. Consider the following code:
+
+        .. code-block:: python
+
+            df['x'] = SeriesFloat64.random(df)
+            df['y'] = df['x']
+            df['different'] = df['y'] != df['x']
+
+        The df['different'] series will be True for (almost) all rows, because the second statement copies
+        the unevaluated expression, not the result of the expression. So at evaluation time the expression
+        will be evaluated twice for each row, for the 'x' column and the 'y' column, giving different
+        results both times. One way to work around this is to materialize the dataframe in its current state
+        (using materialize()), before adding any columns that reference a column that's created with
+        this function.
+        """
+        if is_postgres(base.engine) or is_athena(base.engine):
+            expr_str = 'random()'
+        elif is_bigquery(base.engine):
+            expr_str = 'RAND()'
+        else:
+            raise DatabaseNotSupportedException(base.engine)
+        return cls.get_class_instance(
+            engine=base.engine,
+            base_node=base.base_node,
+            index=base.index,
+            name='random',
+            expression=Expression.construct(expr_str),
+            group_by=None,
+            order_by=[],
+            instance_dtype=cls.dtype
+        )
