@@ -2,18 +2,20 @@
 Copyright 2021 Objectiv B.V.
 """
 import re
-from typing import Union, TYPE_CHECKING, Optional, Pattern
+from typing import Union, TYPE_CHECKING, Optional, Pattern, List
 
 from sqlalchemy.engine import Dialect
 
 from bach.series import Series
-from bach.expression import Expression
+from bach.expression import Expression, AggregateFunctionExpression, join_expressions, get_variable_tokens
+from bach.series.series import WrappedPartition
 from bach.types import StructuredDtype
 from sql_models.constants import DBDialect
+from sql_models.util import DatabaseNotSupportedException, is_bigquery, is_postgres
 
 if TYPE_CHECKING:
     from bach.series import SeriesBoolean
-    from bach import DataFrame
+    from bach import DataFrame, SortColumn, SeriesJson
 
 
 class StringOperation:
@@ -133,6 +135,26 @@ class StringOperation:
         )
         return self._base.copy_override(expression=expr)
 
+    def upper(self) -> 'SeriesString':
+        """
+        converts string values into uppercase.
+
+        :return: SeriesString with all alphabetic characters in uppercase
+        """
+        return self._base.copy_override(
+            expression=Expression.construct('upper({})', self._base)
+        )
+
+    def lower(self) -> 'SeriesString':
+        """
+        converts string values into lowercase.
+
+        :return: SeriesString with all alphabetic characters in lowercase
+        """
+        return self._base.copy_override(
+            expression=Expression.construct('lower({})', self._base)
+        )
+
 
 class SeriesString(Series):
     """
@@ -162,12 +184,22 @@ class SeriesString(Series):
     dtype_aliases = ('text', str)
     supported_db_dtype = {
         DBDialect.POSTGRES: 'text',
+        DBDialect.ATHENA: 'varchar',
         DBDialect.BIGQUERY: 'STRING'
     }
     supported_value_types = (str, type(None))  # NoneType ends up as a string for now
 
     @classmethod
     def supported_literal_to_expression(cls, dialect: Dialect, literal: Expression) -> Expression:
+        # We override the parent class here because strings are really common, and we don't strictly need
+        # to cast them. As all supported databases will interpret a string literal as a string.
+        # Not casting string literals greatly improves the readability of the generated SQL.
+
+        # However, there is an edge case: NULL values should be cast to string. e.g. BigQuery considers a
+        # naked NULL to be INT64. Additionally, we'll always cast variables, just so this keeps working if
+        # the variable get set to `None`
+        if literal.to_sql(dialect=dialect).upper() == 'NULL' or get_variable_tokens([literal]):
+            return super().supported_literal_to_expression(dialect=dialect, literal=literal)
         return literal
 
     @classmethod
@@ -226,3 +258,66 @@ class SeriesString(Series):
 
     def _comparator_operation(self, other, comparator, other_dtypes=tuple(['string'])) -> 'SeriesBoolean':
         return super()._comparator_operation(other, comparator, other_dtypes)
+
+    def to_json_array(self, partition: Optional[WrappedPartition] = None) -> 'SeriesJson':
+        """
+        Aggregate function: Group the values of this Series into a json array
+
+        The order of the values in the array will be based of the order of the values in this Series. If
+        this Series does not have a deterministic sorting, then the values are additionally sorted by the
+        values themselves. A difference in sorting can occur between the resulting array and the values in a
+        Series when the Series contains None/NULL values. Null values will be sorted first when sorting
+        ascending, contrary to sorting inside a DataFrame or a Series where they come last.
+
+        :param partition: The partition to apply, optional.
+        :return: SeriesJson containing an array of strings on each row.
+        """
+        order_by = self.order_by
+        # Add this series as the final column to sort on. If the order_by is deterministic then this won't
+        # change the sorting. If order_by was not deterministic, then this will make it deterministic.
+        from bach import SortColumn
+        order_by += [SortColumn(expression=self.expression, asc=True)]
+
+        order_by_expr = self._get_order_by_expression(order_by)
+        array_agg_expression = AggregateFunctionExpression.construct('array_agg({} {})', self, order_by_expr)
+        if is_postgres(self.engine):
+            expression = Expression.construct('to_jsonb({})', array_agg_expression)
+        elif is_bigquery(self.engine):
+            expression = Expression.construct('to_json_string({})', array_agg_expression)
+        else:
+            raise DatabaseNotSupportedException(self.engine)
+
+        result = self._derived_agg_func(partition, expression)
+        from bach import SeriesJson
+        return result.copy_override_type(SeriesJson)
+
+    @staticmethod
+    def _get_order_by_expression(order_by: List['SortColumn']) -> Expression:
+        """
+        Internal helper function: Convert order_by into an order by expression that is usable inside an
+        array_agg() function
+
+        Note: ordering is slightly different from regular ordering of rows inside a DataFrame or Series, as
+        BigQuery does not support 'asc nulls last' inside aggregations. So we sort 'asc nulls first'.
+        """
+
+        # This logic is partially duplicating DataFrame._get_order_by_clause(), but we can't quite re-use
+        # that as some of the checks there are not applicable (e.g. it's fine to use a column that we are not
+        # grouping on), and the sorting of null values is slightly different.
+        if not order_by:
+            raise ValueError('order_by is empty')
+
+        expressions: List[Expression] = []
+        for sc in order_by:
+            if sc.expression.has_multi_level_expressions:
+                multi_lvl_exprs = [
+                    level_expr for level_expr in sc.expression.data if isinstance(level_expr, Expression)
+                ]
+            else:
+                multi_lvl_exprs = [sc.expression]
+            for level_expr in multi_lvl_exprs:
+                asc_expr = Expression.construct(f"{'asc nulls first' if sc.asc else 'desc nulls last'}")
+                expr = Expression.construct('{} {}', level_expr, asc_expr)
+                expressions.append(expr)
+
+        return Expression.construct('order by {}', join_expressions(expressions))
